# Documentation string. I wonder what ever refers to it?

testString = """
http://www.namshub.org:8080/test/manila/message/create?username=""&password=""&sitename=""&subject="test"&body="test%20test%20test"&bodyType="text/plain"&inResponseTo=""&windowInfo=""&rendererInfo=""
"""

__doc__ = """ZopeFish product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile      # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent    # makes an object stick in the ZODB

#from ZODB.Transaction import Transaction, get_transaction

import OFS.SimpleItem
import OFS.Folder
import Acquisition
import AccessControl.Role

import re
import string
from xmlrpclib import True, False, Server

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addZopeFishForm = HTMLFile('zopefishAdd', globals()) 

def manage_addZopeFish(self, id, title='', REQUEST=None):
    """Add a ZopeFish to a folder."""
    if title == '':
        title = 'ZopeFish!'
    self._setObject(id, ZopeFish(id, title, REQUEST))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

"""Step #2: Provide the class for your ZopeFish object."""

class DocumentControl(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """documentcontrol"""

    default = ""
    current = ""
    ownerName = ""
    ownerPassword = ""
    owned = 0

    manage_main = HTMLFile('leafnodesDisplay', globals()) # Management Interface

    def __init__(self, defaultString, id, parent):
        """__init__"""
        self.default = defaultString
        self.current = defaultString
        self.ownerName = ""
        self.ownerPassword = ""
        self.owned = 0
        self.parent = parent
        self.id = id
        self.title = id

    def objectItems(self):
        """   """
        return [("checkIn", self.checkIn),
                ("get", self.get),
                ("restoreDefault", self.restoreDefault),
                ("set", self.set)]

    def checkIn(self, username, password, siteName):
        """checkIn"""
        return 1

    def get(self, username, password, siteName):
        """get"""
        return self.current

    def restoreDefault(self, username, password, sitename):
        """restoreDefault"""
        self.current = self.default

    def set(self, username, password, sitename, s, windowInfo):
        """set"""
        self.current = s

class Messages(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """messages"""

    manage_main = HTMLFile('leafnodesDisplay', globals()) # Management Interface

    def __init__(self, id, parent):
        """   """
        self.parent = parent
        self.id = id
        self.title = id

    def objectItems(self):
        """   """
        return [("addToStoriesList", self.addToStoriesList),
                ("checkIn", self.checkIn),
                ("isStory", self.isStory),
                ("removeFromStoriesList", self.removeFromStoriesList),
                ("create", self.create),
                ("set", self.set)]

    def addToStoriesList(self, username, password, sitename, msgNum):
        """addToStoriesList"""
        print "addToStoriesList:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", sitename
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def checkIn(self, username, password, siteName, msgNum):
        """checkIn"""
        print "checkIn:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsiteName:", siteName
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def isStory(self, username, password, sitename, msgNum):
        """isStory"""
        print "isStory:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", sitename
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return False

    def removeFromStoriesList(self, username, password, siteName, msgNum):
        """removeFromStoriesList"""
        print "removeFromStoriesList:"
        print "\tusername:", username
        print "\tpassword:", password
        print "\tsitename:", siteName
        print "\tmsgNum  :", msgNum
        print "-----------------"
        return True

    def create(self, username, password, sitename, subject, body,
               bodyType, inResponseTo, windowInfo, rendererInfo):
        """create"""
        print "create:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "\tsubject     :", subject
        print "\tbody        :", body
        print "\tbodyType    :", bodyType
        print "\tinResponseTo:", inResponseTo
        print "\twindowInfo  :", windowInfo
        print "\trendererInfo:", rendererInfo
        print "---------------------"
        headers = self.makeHeaders(subject, bodyType, inResponseTo, rendererInfo, windowInfo)
        msgnum = self.parent.discuss.addNextMessage(body, headers)
        storyInfo = self.parent.getStoryInfo(None, subject, body, msgnum, username)
        return storyInfo

    def set(self, username, password, sitename, msgnum, subject, body,
            bodyType, windowInfo, rendererInfo):
        """set"""
        print "set:"
        print "\tusername    :", username
        print "\tpassword    :", password
        print "\tsitename    :", sitename
        print "\tmsgnum      :", msgnum
        print "\tsubject     :", subject
        print "\tbody        :", body
        print "\tbodyType    :", bodyType
        print "\twindowInfo  :", windowInfo
        print "\trendererInfo:", rendererInfo
        print "---------------------"
        headers = self.parent.discuss.getMessage(msgnum).headers
        self.parent.discuss.addMessage(msgnum, body, headers)
        storyInfo = self.parent.getStoryInfo(None, subject, body, msgnum, username)
        return storyInfo

    def makeHeaders(self, subject, bodyType, inResponseTo, rendererInfo, windowInfo):
        """makeHeaders"""
        headers = {}
        headers['subject'] = subject
        headers['bodyType'] = bodyType
        headers['inResponseTo'] = inResponseTo
        headers['rendererInfo'] = rendererInfo
        headers['windowInfo'] = windowInfo
        return headers

class PikeCompatibility(
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):
    """The Pike Compatible XML-RPC interface"""

    manage_main = HTMLFile('zopefishEdit', globals()) # Management Interface

    def __init__(self, id, parent):
        self.parent = parent
        self.id = id
        self.title = "Pike Compatible XML-RPC Interface"
        self.css = DocumentControl(defaultCss, "css", self.parent)
        self.customPrefs = DocumentControl(defaultCustomPrefs, "customPrefs", self.parent)
        self.javaScript = DocumentControl(defaultJavaScript, "javaScript", self.parent)
        self.joinNowMembersBox = DocumentControl(defaultJoinNowMembersBox, "joinNowMembersBox", self.parent)
        self.message = Messages("message", self.parent)
        self.navigation = DocumentControl(defaultNavigation, "navigation", self.parent)
        self.newsDayTemplate = DocumentControl(defaultNewsDayTemplate, "newsDayTemplate", self.parent)
        self.signOutMembersBox = DocumentControl(defaultSignOutMembersBox, "signOutMembersBox", self.parent)
        self.siteStructure = DocumentControl(defaultSiteStructure, "siteStructure", self.parent)
        self.template = DocumentControl(defaultTemplate, "template", self.parent)

    def objectItems(self):
        """   """
        return [(self.css.id, self.css),
                (self.customPrefs.id, self.customPrefs),
                (self.javaScript.id, self.javaScript),
                (self.joinNowMembersBox.id, self.joinNowMembersBox),
                (self.message.id, self.message),
                (self.navigation.id, self.navigation),
                (self.newsDayTemplate.id, self.newsDayTemplate),
                (self.signOutMembersBox.id, self.signOutMembersBox),
                (self.siteStructure.id, self.siteStructure),
                (self.template.id, self.template)]

class ZopeFishMessage:
    """ZopeFishMessage"""

    msg = ""
    headers = {}

    def __init__(self, theMsg, theHeaders, key, parent):
        """__init__"""
        self.msg = theMsg
        self.headers = theHeaders
        self.key = key
        self.parent = parent

    def editInTextArea(self, REQUEST=None):
        """   """
        pass

    def editInPike(self, REQUEST=None):
        """   """
        storyInfo = self.parent.getStoryInfo(REQUEST, self.headers['subject'], self.msg, self.key)
        siteInfo = self.parent.getSiteInfo(REQUEST)
        self.parent.getPikeServer(REQUEST).pike.editStory(siteInfo, storyInfo)
        if REQUEST is not None:
            return REQUEST.RESPONSE.redirect(self.parent.location + "/discuss/" + self.key)
        else:
            return str(msg)

    def __str__(self):
        """__str__"""
        return zopefishMessageTemplate % (self.msg, self.parent.id, self.key, self.parent.id, self.key)

class MessageCollection(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ):
    """discussion group"""

    #messages = {}
    currentNum = 0

    manage_main = HTMLFile("discussEdit", globals())

    def __init__(self, id, parent):
        """__init__"""
        #self.messages = {}
        self.parent = parent
        self.currentNum = 0
        self.id = id
        self.title = "Message Collection"
        #get_transaction.commit(1)   # get a _p_jar

    def _getNextMessageKey(self):
        nextNum = self.currentNum
        self.currentNum = self.currentNum + 1
        return "msg%06d" % nextNum

    def addNextMessage(self, msg, headers):
        """   """
        msgnum = self._getNextMessageKey()
        self.addMessage(
            msgnum,
            msg,
            headers)
        return msgnum

    def addNextMessageNoHeaders(self, msg):
        """   """
        return self.addNextMessage(msg, {})

    def addMessage(self, key, msg, headers):
        """   """
        msgs = self.__dict__
        msgs[key] = ZopeFishMessage(msg, headers, key, self.parent)
        self.__dict__ = msgs
        self.parent.resync()

    def addMessageNoHeader(self, key, msg):
        """   """
        self.addMessage(key, msg, {})

    def checkMessage(self, key):
        """   """
        if not self.__dict__.has_key(key):
            raise AttributeError

    def index_html(self):
        """call"""
        result = "<html><body>messages:<br>"
        for k in self.__dict__.keys():
            if k[0:3] == "msg":
                result = result + "----------------<br>"
                result = result + "key: " + k + "<br>"
                result = result + "msg: " + str(self.__dict__[k]) + "<br>"
        result = result + "<hr>"
        result = result + "<a href=\"" + self.parent.location + "/discuss/newMessageInPike\">new message</a>"
        result = result + "</body></html>"
        return result

    def newMessageInPike(self, REQUEST=None):
        """   """
        storyInfo = self.parent.getStoryInfo(REQUEST, "New Pike Message", "", "msg000000")
        siteInfo = self.parent.getSiteInfo(REQUEST)
        self.parent.getPikeServer(REQUEST).pike.newStory(siteInfo, storyInfo)

    def getMessage(self, msgnum, REQUEST=None):
        """   """
        self.checkMessage(msgnum)
        msg = self.__dict__[msgnum]
        if REQUEST is not None:
            return str(msg)
        else:
            return msg

    def objectItems(self):
        """objectItems"""
        list = []
        for k in self.__dict__.keys():
            if k[0:3] == "msg":
                list.append((k, self.__dict__[k]))
        return list

class StoryReader:
    """StoryReader"""
    pass

class ZopeFish(
    OFS.Folder.Folder,
    Persistent,            # Make us persistent. Yaah! 
    Acquisition.Implicit,  # Uh, whatever. 
    AccessControl.Role.RoleManager # Security manager. 
    ): 
    """ZopeFish object. 
    
    This is an attempt at writing a product from scratch using mostly 
    Python instead of ZClasses. We'll see how it all goes.
    """
        
    meta_type = 'ZopeFish' # what do people think they're adding? 

    manage_options = ( # what management options are there?
	{'label': 'Edit',       'action': 'manage_main'},
	{'label': 'View',       'action': ''}, # defaults to index_html
	{'label': 'Security',   'action': 'manage_access'},
        ) 

    __ac_permissions__=( # what permissions make sense for us? 
	('View management screens', ('manage_tabs','manage_main')),
	('Change permissions',      ('manage_access',)           ),
	('Change ZopeFishes',       ('manage_edit',)             ),
	('View ZopeFishes',         ('',)                        ),
	)

    def __init__(self, id, title='', REQUEST=None): 
        """initialise a new instance of ZopeFish"""
        self.id = id
        self.title = title
        self.discuss = MessageCollection("discuss", self)
        self.manila = PikeCompatibility("manila", self)

        if REQUEST is not None:
            self.location = REQUEST.URL3 + "/" + self.id
        else:
            self.location = "unknown"

    def resync(self):
        """resync"""
        discuss = self.discuss
        self.discuss = discuss
        manila = self.manila
        self.manila = manila

    def allMessages(self):
        """allMessages"""
        return self.discuss.index_html()

    def objectItems(self):
        """   """
        return [(self.discuss.id, self.discuss), (self.manila.id, self.manila)]

    def getMember(self, REQUEST):
        """   """
        if REQUEST is not None:
            return REQUEST.AUTHENTICATED_USER.getUserName()
        else:
            return "test member"

    def getStoryInfo(self, REQUEST, subject, body, msgNum, member=None):
        """   """
        if member is None:
            thisMember = self.getMember(REQUEST)
        else:
            thisMember = member

        if REQUEST is None:
            url = self.location
        else:
            url = REQUEST.URL2

        # this is a hack.
        if string.find(url, "/discuss") == -1:
            url = url + "/discuss"

        storyInfo = {}
        storyInfo['body'] = body
        storyInfo['member'] = thisMember
        storyInfo['msgnum'] = msgNum
        storyInfo['postTime'] = ""
        storyInfo['lastUpdate'] = ""
        storyInfo['ctReads'] = 0
        storyInfo['subject'] = subject
        storyInfo['responses'] = []
        storyInfo['URL'] = url + "/" + msgNum
        return storyInfo

    def getSiteInfo(self, REQUEST):
        """   """
        host = REQUEST.HTTP_HOST
        i = string.find(host, ":")
        if i != -1:
            host = host[0:i]

        siteInfo = {}
        siteInfo['host'] = host
        siteInfo['port'] = REQUEST.SERVER_PORT
        siteInfo['rpcPath'] = "/" + self.id
        siteInfo['siteName'] = self.id
        siteInfo['displaySiteName'] = self.title
        siteInfo['username'] = self.getMember(REQUEST)
        siteInfo['password'] = ""
        siteInfo['URL'] = REQUEST.URL2
        return siteInfo

    def getPikeServer(self, REQUEST=None):
        """   """
        if REQUEST is not None:
            #print "addr:", REQUEST.REMOTE_ADDR
            return Server("http://" + REQUEST.REMOTE_ADDR + ":81")
        else:
            return Server(myPike)

    index_html = HTMLFile('index', globals()) # View Interface
    
    manage_main = HTMLFile('zopefishEdit', globals()) # Management Interface
    
    def manage_edit(self, title, REQUEST=None): 
        """does this really need a doc string?"""
        self.title = title
	if REQUEST is not None:
	    return MessageDialog(
		title = 'Edited',
		message = "Properties for %s changed." % self.id,
		action = './manage_main',
		)

zopefishMessageTemplate = """
<html>
   <head>
   <title>ZopeFish Message</title>
   </head>
   <body>
   %s
   <hr>
   <a href="/%s/discuss/%s/editInTextArea">edit in web browser</a><br>
   <a href="/%s/discuss/%s/editInPike">edit in pike</a><br>
   this is a ZopeFish site
   </body>
</html>
"""

defaultCss = """
"""

defaultCustomPrefs = """
"""

defaultJavaScript = """
"""

defaultJoinNowMembersBox = """
"""

defaultNavigation = """
"""

defaultNewsDayTemplate = """
"""

defaultSignOutMembersBox = """
"""

defaultSiteStructure = """
"""

defaultTemplate = """
"""

